import pygame
import math
from pygame.locals import *
from spriteanimation import *



__author__="mg"
__date__ ="$May 27, 2013 1:00:47 PM$"

class EnemySprite(pygame.sprite.Sprite):

    def __init__(self, initial_position, speed,hp,boundaries,color=""):
        pygame.sprite.Sprite.__init__(self)

        self.idleImg = pygame.image.load("img/casterMinion{0}Front.png".format(color)).convert_alpha()
        self.image = self.idleImg
        self.hpBar = pygame.Surface((50,5))
        self.rect = self.image.get_rect()
        self.rect.topleft = initial_position
        self.speed = speed
        self.oldSpeed = speed
        self.hasCollision = True
        self.direction = 1
        self.maxHp = hp
        self.hp = hp
        self.disables = {}
        self.hitBy={}
        self.animationFPS = 10
        self.boundaries = boundaries
        self.reachedBottom = False
        self.score = 10
        self.color = color

        #animations
        self.dieAnimation = Animation(Animation.load_sliced_sprites(42,42,"img/casterMinion{0}Die.png".format(color)))
        self.dieAnimation.set_loops(False);
        self.dieAnimation.set_framerate(self.animationFPS)
        self.walkLeftAnimation = Animation(Animation.load_sliced_sprites(42,42,"img/casterMinion{0}WalkLeft.png".format(color)))
        self.walkLeftAnimation.set_framerate(self.animationFPS)
        self.walkRightAnimation = Animation(Animation.load_sliced_sprites(42,42,"img/casterMinion{0}WalkRight.png".format(color)))
        self.walkRightAnimation.set_framerate(self.animationFPS)


    #deal damage (int damage)
    # fuegt diesem gegner schaden zu
    # ret: bool ob toedlicher schaden verursacht wurde
    def dealDamage(self,dmg):
        self.hp -= dmg
        return self.hp <= 0

    def hitWith(self,projectileAtTime):
        self.hitBy.update(projectileAtTime)

    def disable(self,disable):
        self.disables.update(disable)

    def drawHealthBar(self):
        pygame.draw.rect(self.hpBar,(255,0,0),(0,0,50,5))
        pygame.draw.rect(self.hpBar,(0,255,0),(0,0, math.ceil(50.0 * (float(self.hp)/float(self.maxHp))) ,5))
        self.image.blit(self.hpBar, (0,0))


    def update(self,timestamp,stageWidth,time_passed):
        # dead
        print(self.speed)

        if(self.hp<= 0):
            self.image = self.dieAnimation.get_frame(time_passed)
            self.hasCollision = False
            self.speed = 0
            if(self.dieAnimation.finished): self.kill()
        else:

            if(self.speed > 0):
                if(self.direction == 1):
                    self.image = self.walkRightAnimation.get_frame(time_passed)
                else:
                    self.image = self.walkLeftAnimation.get_frame(time_passed)
            else:
                self.image = self.idleImg

            self.drawHealthBar()

        # rooted
        if "root" in self. disables:
            if(self.disables["root"][0] + self.disables["root"][1]*1000 > timestamp):
                self.speed = 0
                print("enemy rooted")
            else:
                print("disabling root")
                self.speed = self.oldSpeed
                del self.disables["root"]

        #slow offset reduction
        if "slow" in self.disables:
            if(self.disables["slow"][0] + self.disables["slow"][1]*1000 > timestamp):
                self.speed = self.oldSpeed * (1-self.disables["slow"][2])
                self.walkLeftAnimation.set_framerate(math.ceil(self.animationFPS*(1-self.disables["slow"][2])))
            else:
                print("disabling slow")
                del self.disables["slow"]
                self.walkLeftAnimation.set_framerate(self.animationFPS)
                self.speed = self.oldSpeed

        #calc offset
        moved_distance = math.ceil(time_passed * self.speed)
        offset = self.direction * moved_distance
        self.rect.left += offset

        #print("{0},{1} vs {2},{3}".format(self.rect.bottom,self.rect.right,self.boundaries.bottom,self.boundaries.right))

        if(self.rect.bottom >= self.boundaries.bottom):
            self.reachedBottom = True



class EnemySpriteTop(pygame.sprite.Sprite):

    def __init__(self, initial_position, speed,hp,boundaries):
        pygame.sprite.Sprite.__init__(self)
        color = "Red"
        self.idleImg = pygame.image.load("img/casterMinion{0}Front.png".format(color)).convert_alpha()
        self.image = self.idleImg
        self.hpBar = pygame.Surface((50,5))
        self.rect = self.image.get_rect()
        self.rect.topleft = (0,0)
        self.speed = speed
        self.oldSpeed = speed
        self.hasCollision = True
        self.direction = 1
        self.maxHp = hp
        self.hp = hp
        self.disables = {}
        self.hitBy={}
        self.animationFPS = 10
        self.boundaries = boundaries
        self.reachedBottom = False
        self.score = 100
        self.color = color

        #animations
        self.dieAnimation = Animation(Animation.load_sliced_sprites(42,42,"img/casterMinion{0}Die.png".format(color)))
        self.dieAnimation.set_loops(False);
        self.dieAnimation.set_framerate(self.animationFPS)
        self.walkLeftAnimation = Animation(Animation.load_sliced_sprites(42,42,"img/casterMinion{0}WalkLeft.png".format(color)))
        self.walkLeftAnimation.set_framerate(self.animationFPS)
        self.walkRightAnimation = Animation(Animation.load_sliced_sprites(42,42,"img/casterMinion{0}WalkRight.png".format(color)))
        self.walkRightAnimation.set_framerate(self.animationFPS)


    #deal damage (int damage)
    # fuegt diesem gegner schaden zu
    # ret: bool ob toedlicher schaden verursacht wurde
    def dealDamage(self,dmg):
        self.hp -= dmg
        return self.hp <= 0

    def hitWith(self,projectileAtTime):
        self.hitBy.update(projectileAtTime)

    def disable(self,disable):
        self.disables.update(disable)

    def drawHealthBar(self):
        pygame.draw.rect(self.hpBar,(255,0,0),(0,0,50,5))
        pygame.draw.rect(self.hpBar,(0,255,0),(0,0, math.ceil(50.0 * (float(self.hp)/float(self.maxHp))) ,5))
        self.image.blit(self.hpBar, (0,0))


    def update(self,timestamp,stageWidth,time_passed):
        # dead
        if(self.hp<= 0):
            self.image = self.dieAnimation.get_frame(time_passed)
            self.hasCollision = False
            self.speed = 0
            if(self.dieAnimation.finished): self.kill()
        else:

            if(self.speed > 0):
                self.image = self.walkRightAnimation.get_frame(time_passed)
            else:
                self.image = self.idleImg

            self.drawHealthBar()

        # rooted
        if "root" in self. disables:
            if(self.disables["root"][0] + self.disables["root"][1]*1000 > timestamp):
                self.speed = 0
                print("enemy rooted")
            else:
                print("disabling root")
                self.speed = self.oldSpeed
                del self.disables["root"]

        #slow offset reduction
        if "slow" in self.disables:
            if(self.disables["slow"][0] + self.disables["slow"][1]*1000 > timestamp):
                self.speed = self.oldSpeed * (1-self.disables["slow"][2])
                self.walkLeftAnimation.set_framerate(math.ceil(self.animationFPS*(1-self.disables["slow"][2])))
            else:
                print("disabling slow")
                del self.disables["slow"]
                self.walkLeftAnimation.set_framerate(self.animationFPS)
                self.speed = self.oldSpeed

        #calc offset
        moved_distance = math.ceil(time_passed * self.speed)
        offset = self.direction * moved_distance
        self.rect.left += moved_distance

        #print("{0},{1} vs {2},{3}".format(self.rect.bottom,self.rect.right,self.boundaries.bottom,self.boundaries.right))

        if(self.rect.left >= self.boundaries.right):
            self.kill()


